from zope.component import getMultiAdapter
from Products.Five.browser import BrowserView
from ZODB.POSException import ConflictError

# This code is "stolen" from singing and dancing, see https://svn.plone.org/svn/collective/collective.dancing/

import logging
logger = logging.getLogger("Plone")

try:
    from BeautifulSoup import BeautifulSoup
    HAS_SOUP = True
except:
    HAS_SOUP = False

plone_html_strip_not_likey = [
    {'id': 'review-history'},
    {'class':'documentActions'},
    {'class':'portalMessage'},
    {'id':'plone-document-byline'},
    {'id':'portlets-below'},
    {'id':'portlets-above'},
    {'class': 'newsletterExclude'},
    ]

def plone_html_strip(html, not_likey=plone_html_strip_not_likey):

    r"""Tries to strip the relevant parts from a Plone HTML page.

    Looks for ``<div id="content">`` and ``<div id="region-content">`` as
    a fallback.

      >>> html = (
      ...     '<html><body><div id="content"><h1 class="documentFirstHeading">'
      ...     'Hi, it\'s me!</h1><p>Wannabe the son of Frankenstein</p>'
      ...     '</div></body></html>')
      >>> plone_html_strip(html)
      u'<h1 class="documentFirstHeading">Hi, it\'s me!</h1><p>Wannabe the son of Frankenstein</p>'
      >>> plone_html_strip('<div id="region-content">Hello, World!</div>')
      u'Hello, World!'

    Will also strip away any ``<div id="review-history">``:

      >>> html = (
      ...     '<div id="region-content">'
      ...     '<div id="review-history">Yesterday</div>Tomorrow</div>')
      >>> plone_html_strip(html)
      u'Tomorrow'

    """

    if not isinstance(html, unicode):
        html = unicode(html, 'UTF-8')

    soup = BeautifulSoup(html)
    content = soup.find('div', attrs={'id': 'content'})
    if content is None:
        content = soup.find('div', attrs=dict({'id': 'region-content'}))

    for attrs in not_likey:
        for item in content.findAll(attrs=attrs):
            item.extract() # meaning: item.bye()
    return content.renderContents(encoding=None) # meaning: as unicode

class Newsletter(BrowserView):
    def title(self):
        return self.context.aq_parent.Title()

    def template(self):
        if not HAS_SOUP:
            logger.info("BeautifulSoup is not installed, not processing of contents is done.")
            return ''

        if hasattr(self.context, 'REQUEST'):
            context = self.context
        else:
            context = self.context.aq_parent

        # Hack to avoid infinite loop due to request parameters
        context.REQUEST.set('restricted_traverse', True)

        try:
            html = context()
        except ConflictError:
            raise
        except:
            # Simple calling does not work if layout
            # is a view, we can try a little harder...
            html = context.restrictedTraverse(context.getLayout())()

        if 'kss' in html:
            return plone_html_strip(html)
        else:
            return html





from zope.interface import implements
from zope.traversing.interfaces import ITraversable
from zope.component import getUtility
from zope.component import adapts

from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.AdvancedQuery import Le, Eq, In

from Acquisition import Acquired
from Acquisition import Implicit

from DateTime import DateTime

from plone.i18n.normalizer.interfaces import IURLNormalizer

from collective.chimpfeed.interfaces import IFeedSettings
from collective.chimpfeed.interfaces import IBrowserLayer
from collective.chimpfeed.vocabularies import feeds_factory


def make_urls(vocabulary):
    filtered = filter(lambda term: term.title, vocabulary or ())
    normalize = getUtility(IURLNormalizer).normalize

    return tuple(
        ("%s.rss" % normalize(term.title), term.value, term.title)
        for term in sorted(
            filtered, key=lambda term: (' : ' in term.title, term.title))
        )


class ItemProxy(Implicit):
    __allow_access_to_unprotected_subobjects__ = 1

    effective_date = Acquired
    Title = Acquired

    def __init__(self, context):
        self.context = context

    @property
    def getText(self):
        # This may raise an `AttributeError`, but that's expected.
        return self.context.aq_inner.restrictedTraverse('@@rss-summary')

    def getCategory(self):
        obj = self.context.aq_inner

        try:
            field = obj.getField('feedCategory')
        except AttributeError:
            return getattr(obj, 'feedCategory', None)

        if field is not None:
            return field.get(obj)


class Feed(Implicit):
    template = ViewPageTemplateFile("feed.pt.xml")

    max_items = 25

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, context, request, name):
        self.context = context
        self.request = request
        self.name = name

    def HEAD(self, *args):
        """Handle HEAD-request."""

        response = self.template()
        self.request.response.setHeader('Content-Length', len(response))

        # Actually return an empty string
        return u""

    def get_items(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        today = DateTime()

        query = In('review_state', ('published', )) & \
            Eq('chimpfeeds', self.name) & \
            Le('feedSchedule', today)

        settings = IFeedSettings(self.context)
        if settings.use_moderation:
            query = query & Eq('feedModerate', True)

        brains = catalog.evalAdvancedQuery(query, (('Date', 'desc'), ))
        objects = tuple(brain.getObject() for brain in brains)
        return tuple(ItemProxy(obj).__of__(obj) for obj in objects)

    def index_html(self):
        """Publish RSS-feed."""

        return self.template()


class Feeds(Implicit):
    __allow_access_to_unprotected_subobjects__ = 1

    def __getitem__(self, name):
        settings = IFeedSettings(self)
        vocabulary = feeds_factory(settings)

        # Use term's titles as the basis for the feed URLs.
        for url, value, title in make_urls(vocabulary):
            if url == name:
                break
        else:
            return KeyError(name)

        return Feed(self, self.REQUEST, value)

    def pretty_title_or_id(self):
        return self.title or self.id


class FeedTraverser(object):
    implements(ITraversable)
    adapts(ISiteRoot, IBrowserLayer)

    def __init__(self, context, request=None):
        self.context = context
        self.request = request

    def traverse(self, name, ignore):
        feeds = Feeds()

        if name:
            return feeds[name]

        return feeds



import greatape

from zope.interface import Interface
from zope import schema

try:
    from zope.component.hooks import getSite
except ImportError:
    from zope.app.component.hooks import getSite

from plone.portlets.interfaces import IPortletDataProvider

from collective.chimpfeed.permissions import MODERATE_PERMISSION
from collective.chimpfeed.permissions import SCHEDULE_PERMISSION
from collective.chimpfeed import MessageFactory as _
from collective.chimpfeed import logger


def check_api_key(api_key):
    if not api_key:
        return True

    api = greatape.MailChimp(api_key)

    try:
        api(method="ping")
    except greatape.MailChimpError:
        pass
    except ValueError, exc:
        logger.warn(exc)
    else:
        return True

    return False


class IApiUtility(Interface):
    """ Api utuility """


class IGroupSorter(Interface):
    def key(name, items):
        """Return sorting key for group with the provided name and
        items."""


class INameSplitter(Interface):
    def split_name(fullname):
        """Return (fname, lname) tuple."""


class IBrowserLayer(Interface):
    """Theme-specific browser-layer."""


class IFeedSettings(Interface):
    mailchimp_api_key = schema.TextLine(
        title=_(u"MailChimp API key"),
        description=_(u"This key is associated to your "
                      u"account with MailChimp."),
        required=False,
        default=u"",
        constraint=check_api_key,
    )

    mailinglist = schema.Choice(
        title=_(u"Interest group list"),
        description=_(u"Use this field to automatically "
                      u"pull interest groups from a list. "
                      u"The interest groups are "
                      u"added to the list of feeds "
                      "and kept up to date every hour."
                      ),
        required=False,
        vocabulary="collective.chimpfeed.vocabularies.Lists",
    )

    ignored_groupings = schema.Tuple(
        title=_(u"Ignored interest groupings"),
        description=_(u"Use this field to limit the interest "
                      u"groups pulled automatically from the list "
                      u"to specific groupings."
                      ),
        required=False,
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.InterestGroupings",
        ),
    )

    feeds = schema.List(
        title=_(u"Feeds"),
        description=_(u"The strings listed in this field "
                      u"are available for tagging on "
                      u"applicable site content. Please use "
                      u"one value per line. An RSS-feed "
                      u"is automatically available for each "
                      u"string (links are shown below). Note "
                      u"that it's possible to select a "
                      u"subscription list (see above) "
                      u"and pull additional strings from "
                      u"its interest groups."
                      ),
        required=False,
        value_type=schema.TextLine(),
    )

    categories = schema.List(
        title=_(u"Categories"),
        description=_(u"List the available feed categories."),
        required=False,
        value_type=schema.TextLine(),
    )

    use_moderation = schema.Bool(
        title=_(u'Require moderation'),
        description=_(u'Select this option to enable content '
                      u'moderation.'),
        required=False,
    )

    bump_date_on_moderation = schema.Bool(
        title=_(u'Bump date on moderation'),
        description=_(u"Select this option to automatically "
                      u"bump the schedule date to today's date, "
                      u"if it is unset or set to an earlier date."),
        required=False,
        default=True,
    )

    show_name = schema.Bool(
        title=_(u'Show name on subscription form'),
        required=False,
        default=True,
    )

    enable_select_all = schema.Bool(
        title=_(u'Enable select all on subscription form'),
        required=False,
        default=True,
    )

    interests_required = schema.Bool(
        title=_(u'Require one or more interests to subscribe'),
        required=False,
        default=False,
    )


class ISubscriptionFormSettings(Interface):
    mailinglist = schema.Choice(
        title=_(u"Mailinglist"),
        description=_(u"Select a mailinglist for this portlet."),
        vocabulary="collective.chimpfeed.vocabularies.Lists",
        required=True,
    )

    interest_groupings = schema.Tuple(
        title=_(u"Interest groups"),
        description=_(u"Select interest groups. "
                      u"Note that all interests within the selected "
                      u"groups will appear in the portlet."),
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.InterestGroupings",
        ),
        required=False,
    )

    interest_groups = schema.Tuple(
        title=_(u"Interests"),
        description=_(u"Select interests."),
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.InterestGroups",
        ),
        required=False,
    )

    preselected_interest_groups = schema.Tuple(
        title=_(u"Preselected interests."),
        description=_(u"If interests is selected here, the interests groups will be preselected, "
                      u"and the user will not be able to select these."),
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.InterestGroups",
        ),
        required=False,
    )


class ICampaignPortlet(IPortletDataProvider):
    mailinglist = schema.Choice(
        title=_(u"Mailinglist"),
        description=_(u"Select a mailinglist for this portlet."),
        vocabulary="collective.chimpfeed.vocabularies.Lists",
        required=True,
    )

    template = schema.Choice(
        title=_(u"Template"),
        description=_(u"Select a campaign template."),
        required=False,
        default=u"",
        vocabulary="collective.chimpfeed.vocabularies.Templates",
    )

    section = schema.TextLine(
        title=_(u"Section"),
        description=_(u"In MailChimp, templates have multiple sections, "
                      u"each identified by a string. When the "
                      u"campaign is created, we need to know where to "
                      u"put the dynamically created newsletter content."),
        default=u"std_content00",
        required=True,
    )

    subject = schema.TextLine(
        title=_(u"Subject"),
        description=_(u"The newsletter subject."),
        required=False,
        default=u"",
    )

    start = schema.Date(
        title=_(u"Start date"),
        description=_(u"Include items published after this date."),
        required=False,
        default=None,
    )

    image = schema.TextLine(
        title=_(u"Image field"),
        description=_(u"The name of the content field that contains "
                      u"the content lead image. Note that it does not "
                      u"need to exist for all content."),
        required=False,
        default=u"image",
    )

    scale = schema.TextLine(
        title=_(u"Image scale"),
        description=_(u"The name of the image scale."),
        required=False,
        default=u"thumb",
    )


class INewsletterPortlet(IPortletDataProvider):
    mailinglist = schema.Choice(
        title=_(u"Mailinglist"),
        description=_(u"Select a mailinglist for this portlet."),
        vocabulary="collective.chimpfeed.vocabularies.Lists",
        required=True,
    )

    interest_groups = schema.Tuple(
        title=_(u"Interests"),
        description=_(u"Select interests."),
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.InterestGroups",
        ),
        required=False,
    )

    select_interest_groups = schema.Bool(
        title=_(u'Select interest groups before sending'),
        description=_(u'Select this option if the interest groups should be selected '
                      u'before the newsletter is sent.'),
        required=False,
        default=False,
    )

    template = schema.Choice(
        title=_(u"Template"),
        description=_(u"Select a campaign template."),
        required=False,
        default=u"",
        vocabulary="collective.chimpfeed.vocabularies.Templates",
    )

    section = schema.TextLine(
        title=_(u"Section"),
        description=_(u"In MailChimp, templates have multiple sections, "
                      u"each identified by a string. When the "
                      u"campaign is created, we need to know where to "
                      u"put the dynamically created newsletter content."),
        default=u"std_content00",
        required=True,
    )

    subject = schema.TextLine(
        title=_(u"Subject"),
        description=_(u"The newsletter subject."),
        required=False,
        default=u"",
    )


class IModerationPortlet(IPortletDataProvider):
    pass


class ISubscriptionPortlet(IPortletDataProvider, ISubscriptionFormSettings):
    heading = schema.TextLine(
        title=_(u"Title"),
        description=_(u"Provide a title for the portlet."),
        required=True,
        default=u"Sign up",
    )

    description = schema.Text(
        title=_(u"Description"),
        description=_(u"Enter a description of your newsletter."),
        required=False,
        default=u"Join our newsletter and stay updated!",
    )


class IControlPanel(IFeedSettings):
    urls = schema.Tuple(
        title=_(u"RSS"),
        description=_(u"This field lists an RSS-feed for each of "
                      u"the available feed strings (including "
                      u"those that are automatically pulled from "
                      u"interest groups."),
        required=False,
        value_type=schema.ASCIILine(),
    )

    lists = schema.Tuple(
        title=_(u"Lists"),
        description=_(u"This listing shows a link to a "
                      u"subscription form for each of the defined "
                      u"lists in your account."),
        required=False,
        value_type=schema.ASCIILine(),
    )


class IFeedControl(Interface):
    """Feed control settings."""

    feeds = schema.Set(
        title=_(u"Feeds"),
        description=_(u"Select one or more items from this list "
                      u"to include in the corresponding feed."),
        required=False,
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.Feeds",
        )
    )

    feedCategory = schema.Choice(
        title=_(u"Feed category"),
        description=_(u"Please select a category for this item."),
        required=False,
        default=None,
        vocabulary="collective.chimpfeed.vocabularies.Categories",
    )

    feedSchedule = schema.Date(
        title=_(u'Feed date'),
        description=_(u'If this date is set, the content will only be '
                      u'included in Mailchimp-based feeds from this date on. '
                      u'Otherwise, the "Publishing date" is used.'),
        required=False,
    )

    feedModerate = schema.Bool(
        title=_(u'Feed moderation'),
        description=_(u'Select this option to approve item.'),
        required=False,
    )


try:
    from plone.directives.form import Schema
    from plone.supermodel.model import Fieldset
    from plone.supermodel.interfaces import FIELDSETS_KEY
    from plone.autoform.interfaces import WRITE_PERMISSIONS_KEY, WIDGETS_KEY
    from plone.autoform.interfaces import OMITTED_KEY
    from plone.autoform.interfaces import IAutoExtensibleForm
except ImportError:
    pass
else:
    class IFeedControl(Schema, IFeedControl):
        """Form-enabled feed control settings."""

    class moderation_enabled:
        def __nonzero__(self):
            context = getSite()
            settings = IFeedSettings(context)
            return not settings.use_moderation

    moderation_enabled = moderation_enabled()

    class settings_available:
        def __nonzero__(self):
            context = getSite()
            return IFeedSettings(context, None) is None

    settings_available = settings_available()

    IFeedControl.setTaggedValue(
        WRITE_PERMISSIONS_KEY, {
            'feedModerate': MODERATE_PERMISSION,
            'feedSchedule': SCHEDULE_PERMISSION
        },
    )

    IFeedControl.setTaggedValue(
        OMITTED_KEY, (
            (IAutoExtensibleForm, 'feeds', settings_available),
            (IAutoExtensibleForm, 'feedModerate', settings_available),
            (IAutoExtensibleForm, 'feedSchedule', settings_available),
            (IAutoExtensibleForm, 'feedModerate', moderation_enabled),
        )
    )

    IFeedControl.setTaggedValue(
        FIELDSETS_KEY,
        [Fieldset(
            'dates',
            fields=['feedSchedule', 'feedModerate'],
            label=_(u"Dates")),
         Fieldset(
             'categorization',
             fields=['feeds'],
             label=_(u"Categorization")),
         ]
    )

    IFeedControl.setTaggedValue(
        WIDGETS_KEY,
        {'feeds':
         'z3c.form.browser.checkbox.CheckBoxFieldWidget'}
    )


class IGroupExtras(Interface):
    """ """
    def items(self, group, start, until=None):
        """ return extra items to include in feed.
        """



import os

from plone.memoize.instance import memoizedproperty

module_path = os.path.dirname(__file__)


class GenericNameSplitter:
    @staticmethod
    def split_name(name):
        names = filter(None, name.split(u' '))

        if len(names) < 2:
            return name, u''

        if len(names) == 2:
            return names

        last = names[-1]
        return u' '.join(names[:-2]), '%s %s' % (names[-2], last)


class DanishNameSplitter(object):
    @memoizedproperty
    def given_names(self):
        path = os.path.join(module_path, 'names', 'da')
        return open(os.path.join(path, 'male.txt'), 'r').read().\
               decode('utf-8').split(u'\n') + \
               open(os.path.join(path, 'female.txt'), 'r').read().\
               decode('utf-8').split(u'\n')

    def split_name(self, name):
        names = filter(None, name.split(u' '))

        if len(names) < 2:
            return name, u""

        last = names[-1]

        if len(names) == 2 or not last.endswith(u'sen') or \
               names[-2] in self.given_names:
            return u" ".join(names[:-1]), last

        return u" ".join(names[:-2]), u"%s %s" % (names[-2], last)



from zope.formlib import form
from zope.interface import implements
from zope.interface import alsoProvides
from zope.interface import noLongerProvides
from zope.component import getUtility
from plone.i18n.normalizer.interfaces import IIDNormalizer

try:
    from plone.z3cform.interfaces import IWrappedForm
except ImportError:
    IWrappedForm = None
    from plone.z3cform.z2 import switch_on

try:
    from plone.app.z3cform.interfaces import IPloneFormLayer
except:
    from Products.CMFDefault.interfaces import ICMFDefaultSkin as \
         IPloneFormLayer

from z3c.form.interfaces import IFormLayer
from plone.app.portlets.portlets import base

try:
    from five.formlib.formbase import FormBase
except ImportError:
    from Products.Five.formlib.formbase import FormBase

from zope.security import checkPermission

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFDefault.interfaces import ICMFDefaultSkin

from collective.chimpfeed.form import CampaignForm
from collective.chimpfeed.form import NewsletterForm
from collective.chimpfeed.form import SubscribeForm
from collective.chimpfeed.form import ModerationForm
from collective.chimpfeed.interfaces import ISubscriptionPortlet
from collective.chimpfeed.interfaces import IModerationPortlet
from collective.chimpfeed.interfaces import ICampaignPortlet
from collective.chimpfeed.interfaces import INewsletterPortlet
from collective.chimpfeed import MessageFactory as _


class CampaignPortletAssignment(base.Assignment):
    implements(ICampaignPortlet)

    heading = _(u"Campaign scheduling")
    description = _(u"Send or schedule a newsletter campaign.")

    title = _(u"Campaign portlet")

    start = None
    section = u"std_content00"

    def __init__(self, **kwargs):
        for name, value in kwargs.items():
            self.__dict__[name] = value

class NewsletterPortletAssignment(base.Assignment):
    implements(INewsletterPortlet)

    heading = _(u"Static campaign scheduling")
    description = _(u"Send or schedule a newsletter campaign based on the current context.")

    title = _(u"Newsletter portlet")

    maillinglist = ''
    interest_groups = tuple()
    select_interest_groups = False
    template = ''
    section = u"std_content00"
    subject = ''

    def __init__(self, **kwargs):
        for name, value in kwargs.items():
            self.__dict__[name] = value


class ModerationPortletAssignment(base.Assignment):
    implements(IModerationPortlet)

    heading = _(u"Feed moderation")
    description = _(u"Use this form to moderate content.")

    title = _(u"Moderation portlet")


class SubscriptionPortletAssignment(base.Assignment):
    implements(ISubscriptionPortlet)

    description = None

    def __init__(self, **kwargs):
        for name, value in kwargs.items():
            self.__dict__[name] = value

    @property
    def title(self):
        return _("Subscribe: ${title}", mapping={'title': self.heading})


class FormPortletRenderer(base.Renderer):
    render = ViewPageTemplateFile('portlet.pt')

    name = ""

    @property
    def css_class(self):
        """Generate a CSS class from the portlet header
        """
        try:
            header = self.data.heading
            normalizer = getUtility(IIDNormalizer)
            return "portlet-%s" % normalizer.normalize(header)
        except AttributeError:
            return ''

    def render_form(self):
        provided = IPloneFormLayer.providedBy(self.request)
        noLongerProvides(self.request, IPloneFormLayer)
        alsoProvides(self.request, IFormLayer)
        try:
            form = self.create_form()
            if IWrappedForm is None:
                switch_on(self)
                alsoProvides(self.request, ICMFDefaultSkin)
            else:
                alsoProvides(form, IWrappedForm)
            form.update()
            return form.render()
        finally:
            noLongerProvides(self.request, IFormLayer)
            if provided:
                alsoProvides(self.request, IPloneFormLayer)


class ModerationPortletRenderer(FormPortletRenderer):
    name = "Moderation"

    @property
    def available(self):
        return checkPermission("chimpfeed.Moderate", self.context)

    def create_form(self):
        context = self.data.__of__(self.context)
        return ModerationForm(context, self.request)


class SubscriptionPortletRenderer(FormPortletRenderer):
    name = "Subscription"

    def create_form(self):
        context = self.data.__of__(self.context)
        return SubscribeForm(context, self.request)


class CampaignPortletRenderer(FormPortletRenderer):
    name = "Campaign"

    @property
    def available(self):
        return checkPermission('chimpfeed.Campaign', self.context)

    def create_form(self):
        context = self.data.__of__(self.context)
        return CampaignForm(context, self.request)

class NewsletterPortletRenderer(FormPortletRenderer):
    name = "Newsletter"

    @property
    def available(self):
        return checkPermission('chimpfeed.Campaign', self.context)

    def create_form(self):
        context = self.data.__of__(self.context)
        return NewsletterForm(context, self.request)


class CampaignPortletAddForm(base.AddForm):
    label = _(u"Add campaign portlet")
    form_fields = form.Fields(ICampaignPortlet)

    # Let's not ask the user to name a start date on adding the
    # portlet.
    form_fields = form_fields.omit('start')

    def create(self, data):
        return CampaignPortletAssignment(**data)


class CampaignPortletEditForm(base.EditForm):
    label = _(u"Edit campaign portlet")
    form_fields = form.Fields(ICampaignPortlet)


class NewsletterPortletAddForm(base.AddForm):
    label = _(u"Add newsletter portlet")
    form_fields = form.Fields(INewsletterPortlet)

    def create(self, data):
        return NewsletterPortletAssignment(**data)


class NewsletterPortletEditForm(base.EditForm):
    label = _(u"Edit newsletter portlet")
    form_fields = form.Fields(INewsletterPortlet)


class ModerationPortletAddForm(base.NullAddForm):
    def create(self):
        return ModerationPortletAssignment()


class SubscriptionPortletForm(FormBase):
    name = "Subscribe"

    form_fields = form.Fields(ISubscriptionPortlet)
    description = _(u"This portlet shows a subscription form.")

    def setUpWidgets(self, **kwargs):
        super(SubscriptionPortletForm, self).setUpWidgets(**kwargs)

        widget = self.widgets['mailinglist']
        if len(widget.vocabulary):

            mailinglist = ISubscriptionPortlet.providedBy(self.context) and \
                          self.context.mailinglist or widget.hasInput() \
                          and widget.getInputValue()

            if mailinglist:
                pass
            else:
                # Make a default choice
                token = tuple(widget.vocabulary)[-1].token
                widget.setRenderedValue(token)
        else:
            self.widgets['interest_groups'].hidden = True
            self.widgets['interest_groupings'].hidden = True


class SubscriptionPortletAddForm(SubscriptionPortletForm, base.AddForm):
    label = _(u"Add Subscription Portlet")

    def create(self, data):
        return SubscriptionPortletAssignment(**data)


class SubscriptionPortletEditForm(SubscriptionPortletForm, base.EditForm):
    label = _(u"Edit Subscription Portlet")



import os
import re
import time
import datetime
import math
import greatape
import cgi
import urllib
import pprint

from Products.statusmessages.interfaces import IStatusMessage
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.i18nl10n import ulocalized_time
from Products.CMFPlone.interfaces import IPloneSiteRoot

from Acquisition import Implicit, ImplicitAcquisitionWrapper
from ComputedAttribute import ComputedAttribute
from zExceptions import BadRequest

from ZODB.utils import u64
from DateTime import DateTime

from plone.memoize.ram import cache
from plone.memoize import view
from plone.memoize.instance import memoizedproperty
from plone.memoize.volatile import DontCache
from plone.memoize.forever import memoize as forever
from plone.z3cform.layout import wrap_form

from zope.i18n import negotiate
from zope.i18n import translate
from zope.i18n.interfaces import ITranslationDomain
from zope.component import queryUtility, getUtility, getMultiAdapter
from zope.component.hooks import getSite
from zope.interface import alsoProvides
from zope.interface import Interface
from zope.interface import implements
from zope.schema.vocabulary import SimpleVocabulary
from zope.schema.vocabulary import SimpleTerm
from zope.schema import ValidationError
from zope import schema
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile

from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.widget import SequenceWidget
from z3c.form.widget import FieldWidget
from z3c.form.interfaces import IErrorViewSnippet
from z3c.form.interfaces import IActions
from z3c.form.interfaces import IWidgets
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form.browser.checkbox import SingleCheckBoxFieldWidget
from z3c.form.interfaces import IWidget
from z3c.form.interfaces import HIDDEN_MODE

try:
    from z3c.form.interfaces import NO_VALUE
except ImportError:
    from z3c.form.interfaces import NOVALUE as NO_VALUE

from collective.chimpfeed import MessageFactory as _
from collective.chimpfeed import logger
from collective.chimpfeed.interfaces import IFeedSettings
from collective.chimpfeed.interfaces import INameSplitter
from collective.chimpfeed.interfaces import ICampaignPortlet
from collective.chimpfeed.interfaces import INewsletterPortlet
from collective.chimpfeed.interfaces import ISubscriptionFormSettings
from collective.chimpfeed.interfaces import IApiUtility
from collective.chimpfeed.vocabularies import interest_groups_factory
from collective.chimpfeed.vocabularies import InterestGroupVocabulary
from collective.chimpfeed.splitters import GenericNameSplitter

re_email = re.compile(r"^(\w&.%#$&'\*+-/=?^_`{}|~]+!)*[\w&.%#$&'\*+-/=?^_"
                      r"`{}|~]+@(([0-9a-z]([0-9a-z-]*[0-9a-z])?\.)+[a-z]{2,6}"
                      r"|([0-9]{1,3}\.){3}[0-9]{1,3})$", re.IGNORECASE)


def create_groupings(groups):
    groupings = {}
    for grouping_id, name, interest_id in groups:
        groupings.setdefault(grouping_id, []).append(name)
    return groupings


def cache_on_get_for_an_hour(method, self):
    if self.request['REQUEST_METHOD'] != 'GET':
        raise DontCache

    return (
        time.time() // (60 * 60),
        self.id,
        self.request.get('list_id'),
        self.request.get('success')
    )


def is_email(value):
    if re_email.match(value):
        return True

    raise EmailValidationError(value)


class EmailValidationError(ValidationError):
    __doc__ = _(u"Not a valid e-mail address.")


def interests_required(value):
    settings = IFeedSettings(getSite())
    if settings.interests_required and not value:
        raise InterestsRequiredValidationError(value)
    return True


class InterestsRequiredValidationError(ValidationError):
    __doc__ = _(u"Please select at least one interest.")


class ICampaign(ICampaignPortlet):
    """Note that most fields are inherited from the portlet."""

    limit = schema.Bool(
        title=_(u"Limit"),
        description=_(u"Include scheduled items up until today's date only."),
        default=True,
        required=False,
    )

    filtering = schema.Bool(
        title=_(u"Apply filtering markup"),
        description=_(u"Select this option to apply the markup "
                      u"required for automatic interest group "
                      u"filtering."),
        default=True,
        required=False,
    )

    create_draft = schema.Bool(
        title=_(u"Create new campaigns as drafts"),
        required=False,
        default=True
    )

    schedule = schema.Datetime(
        title=_(u"Schedule date"),
        description=_(u"If provided, item will be scheduled to be sent "
                      u"at this time."),
        required=False,
        default=None,
    )


class INewsletter(INewsletterPortlet):
    """Note that most fields are inherited from the portlet."""

    interests = schema.Tuple(
        title=_(u"Select interest groups"),
        description=_(u"If no interest groups are selected the predefined "
                      u"groups will be used, which were specified during the "
                      u"creation of the portlet"),
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.InterestGroups",
        ),
        required=False,
    )

    create_draft = schema.Bool(
        title=_(u"Create new campaigns as drafts"),
        required=False,
        default=True
    )

    schedule = schema.Datetime(
        title=_(u"Schedule date"),
        description=_(u"If provided, item will be scheduled to be sent "
                      u"at this time."),
        required=False,
        default=None,
    )


class IModeration(Interface):
    items = schema.Tuple(
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.ScheduledItems",
        ),
        required=False,
    )


class ISubscription(Interface):
    email = schema.TextLine(
        title=_(u"E-mail"),
        required=True,
        constraint=is_email,
    )

    interests = schema.Tuple(
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.InterestGroups",
        ),
        required=False,
        constraint=interests_required,
    )


class ModerationWidget(SequenceWidget):
    template = ViewPageTemplateFile("moderate.pt")

    @classmethod
    def factory(cls, field, request):
        return FieldWidget(field, cls(request))

    def render(self):
        return self.template()

    def _localize_time(self, time, long_format):
        util = getToolByName(self.context, 'translation_service')

        return util.ulocalized_time(
            time, long_format=long_format,
            context=self.context, request=self.request,
            domain='plonelocales'
        ).lstrip('0')

    @memoizedproperty
    def entries(self):
        entries = []
        catalog = self.context.portal_catalog

        for term in self.terms:
            rid = term.value
            entry = catalog.getMetadataForRID(rid)
            date = entry['feedSchedule']
            if date is not None:
                # Must be a `DateTime`, even for Dexterity-based
                # content.
                if isinstance(date, DateTime):
                    entry['id'] = rid
                    entries.append(entry)

        return entries

    @memoizedproperty
    def action_required(self):
        for entry in self.entries:
            if not entry['feedModerate']:
                return True

    @property
    def groups(self):
        last = None
        groups = []
        entries = []
        today = DateTime()
        today = DateTime(today.year(), today.month(), today.day())

        for entry in self.entries:
            date = entry['feedSchedule']
            days = int(math.floor(date - today))
            if days != last:
                entries = []

                # To-Do: Use Plone's date formatters
                if days == -1:
                    name = _(u"Today")
                else:
                    abbr = date.strftime("%a")
                    name = translate(
                        'weekday_%s' % abbr.lower(),
                        domain="plonelocales",
                        context=self.request
                    ).capitalize()

                    if days < 0 or days >= 7:
                        name = _(u"${subject} ${date}", mapping={
                            'subject': name,
                            'date': self._localize_time(date, False),
                        })

                groups.append({
                    'date': name,
                    'entries': entries,
                })

            last = days
            entries.append(entry)

        return groups


class InterestsWidget(SequenceWidget):
    template = ViewPageTemplateFile("interests.pt")

    @cache(cache_on_get_for_an_hour)
    def render(self):
        return self.template()

    @classmethod
    def factory(cls, field, request):
        return FieldWidget(field, cls(request))

    @property
    def vocabulary(self):
        settings = IFeedSettings(self.context)
        return ImplicitAcquisitionWrapper(
            interest_groups_factory, settings)

    @memoizedproperty
    def groupings(self):
        utility = getUtility(IApiUtility, context=self.context)
        wrapped = utility.__of__(self.context)
        return tuple(wrapped.get_groupings())

    def renderInterestGroups(self):
        groups = create_groupings(self.context.interest_groups)

        terms = []
        for grouping in self.groupings:
            grouping_id = grouping['id']
            names = groups.get(grouping_id, ())
            for group in grouping['groups']:
                name = group['name']
                if name in names:
                    terms.append(self.vocabulary.get_term_for_group(
                        group, grouping
                    ))

        return self.renderChoices(terms)

    def renderInterestGroupings(self):
        filtered = (
            grouping for grouping in self.groupings
            if grouping['id'] in self.context.interest_groupings
        )

        rendered = []

        for grouping in filtered:
            terms = tuple(self.vocabulary.get_terms_for_grouping(
                grouping
            ))

            # Create label from the grouping name
            label = cgi.escape(grouping['name'])
            result = self.renderChoices(terms, label, grouping['id'])
            rendered.append(result)

        return u"\n".join(rendered)

    def renderChoices(self, terms, label=None, grouping_id=None):
        choice = schema.Choice(
            vocabulary=SimpleVocabulary(terms),
        )

        field = self.field.bind(self.context)
        field.value_type = choice

        widget = CheckBoxFieldWidget(field, self.request)
        widget.mode = self.mode
        widget.name = self.name
        widget.id = self.id

        if grouping_id is not None:
            widget.id += "-%s" % grouping_id

        widget.update()
        result = widget.render()

        if label is not None and self.mode != HIDDEN_MODE:
            result = (
                u'<fieldset class="interest-group">'
                u'<legend>%s</legend>%s</fieldset>') % (label, result)

        return result


class SchemaErrorSnippet(object):
    implements(IErrorViewSnippet)

    def __init__(self, error, request, widget, field, form, content):
        self.error = error
        self.widget = widget
        self.message = error.__doc__

    def update(self):
        pass

    def render(self):
        return _(self.error.__doc__)


class BaseForm(form.Form):
    ignoreContext = True

    @property
    def prefix(self):
        # Make sure form prefix is unique
        return 'form-%d-%d.' % (
            u64(self.context._p_oid),
            int(self.context._p_mtime) % 10000
        )


class BaseCampaignForm(BaseForm):
    def createCampaign(self, api_key, method, subject,
                       create_draft, schedule, rendered, next_url,
                       segment_opts={}):
        try:
            if not rendered.strip():
                IStatusMessage(self.request).addStatusMessage(
                    _(u"No content found; newsletter not created."),
                    "info"
                )

                return

            if api_key:
                api = greatape.MailChimp(api_key, debug=False)

                try:
                    for entry in api(method="lists"):
                        if entry['id'] == self.context.mailinglist:
                            break
                    else:
                        IStatusMessage(self.request).addStatusMessage(
                            _(u"Mailinglist not found."),
                            "error"
                        )

                        return

                    if self.context.template:
                        section = 'html_%s' % self.context.section
                    else:
                        section = 'html'

                    args = {}

                    args['method'] = method
                    args['type'] = 'regular'
                    args['content'] = {section: rendered}
                    if segment_opts:
                        args['segment_opts'] = segment_opts

                    options = {}
                    options['subject'] = subject.encode("utf-8") or \
                        entry['default_subject']
                    options['from_email'] = entry['default_from_email']
                    options['from_name'] = \
                        entry['default_from_name'].encode("utf-8")
                    options['to_email'] = 0
                    options['list_id'] = self.context.mailinglist
                    options['template_id'] = self.context.template or None
                    options['generate_text'] = True

                    args['options'] = options

                    result = api(**args)

                    if result:
                        if not create_draft:
                            if schedule:
                                # Apply local time zone to get GMT
                                schedule = schedule + datetime.timedelta(
                                    seconds=time.timezone
                                )

                                # Format time
                                schedule_time = time.strftime(
                                    "%Y-%m-%d %H:%M:%S", schedule.utctimetuple()
                                )

                                schedule = api(
                                    method="campaignSchedule",
                                    cid=result,
                                    schedule_time=schedule_time,
                                )

                                if not schedule:
                                    IStatusMessage(self.request).addStatusMessage(
                                        _(u"Campaign created, but not scheduled."),
                                        "error"
                                    )

                                    return
                            else:
                                api(method="campaignSendNow",
                                    cid=result)

                        next_url = self.context.portal_url() + \
                            "/@@chimpfeed-content?cid=%s" % result

                except greatape.MailChimpError, exc:
                    IStatusMessage(self.request).addStatusMessage(
                        _(u"Unable to process request: ${message}",
                          mapping={'message': exc.msg}), "error"
                    )
                    logger.warn(exc.msg)
                    logger.warn(pprint.pformat(args))
                    return

                return bool(result)
        finally:
            self.request.response.redirect(next_url)


class CampaignForm(BaseCampaignForm):
    fields = field.Fields(
        ICampaign['start'],
        ICampaign['limit'],
        ICampaign['filtering'],
        ICampaign['subject'],
        ICampaign['create_draft'],
        ICampaign['schedule'],
    )

    fields['limit'].widgetFactory = SingleCheckBoxFieldWidget
    fields['filtering'].widgetFactory = SingleCheckBoxFieldWidget
    fields['create_draft'].widgetFactory = SingleCheckBoxFieldWidget

    ignoreContext = True

    @button.buttonAndHandler(_(u'Preview'))
    def handlePreview(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        url = self.context.aq_parent.absolute_url()
        params = self.makeParams(**data)
        return self.request.response.redirect(
            url + "/@@chimpfeed-preview?%s" % urllib.urlencode(params))

    def makeParams(self, start=None, limit=False,
                   filtering=False, **extra):
        today = datetime.datetime.now()

        return dict(
            start=start.isoformat(),
            filtering=filtering and '1' or "",
            until=limit and today.isoformat() or "",
            image=self.context.image,
            scale=self.context.scale,
        )

    @button.buttonAndHandler(_(u'Create'))
    def handleCreate(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        success = self.process("campaignCreate", **data)

        if success:
            IStatusMessage(self.request).addStatusMessage(
                _(u"Campaign created."),
                "info"
            )

            # Set date to tomorrow's date.
            self.context.start = datetime.date.today() + \
                datetime.timedelta(days=1)

    def process(self, method, subject=None,
                create_draft=False, schedule=None, **data):
        settings = IFeedSettings(self.context)
        api_key = settings.mailchimp_api_key

        view = getMultiAdapter(
            (self.context.aq_parent, self.request), name="chimpfeed-campaign"
        )

        params = self.makeParams(**data)
        rendered = view.template(**params).encode('utf-8')
        next_url = self.request.get('HTTP_REFERER') or self.action

        segments = view.getSegments(**params).items()
        if segments:
            segment_opts = {
                'match': 'all',
                'conditions':
                [{'field': 'interests-%s' % groupingid,
                  'op':'one',
                  'value': groupids}
                 for groupingid, groupids in segments[:10]
                 ]}

            if len(segments) > 10:
                count = len(segments) - 10
                IStatusMessage(self.request).addStatusMessage(
                    _(u"%d segments were ignored "
                      u"(Mailchimp limits this to 10)." % count),
                    "warning"
                )
        else:
            segment_opts = {}

        return self.createCampaign(api_key, method, subject, create_draft,
                                   schedule, rendered, next_url, segment_opts)

    def update(self):
        super(CampaignForm, self).update()

        today = datetime.date.today()

        date = self.context.start
        if date is None:
            date = today

        start = self.widgets['start']
        start.value = (date.year, date.month, date.day)

        subject = self.widgets['subject']
        if not subject.value:
            value = self.context.subject or \
                self.context.Title().decode('utf-8')

            subject.value = _(
                u"${subject} ${date}", mapping={
                'subject': value, 'date': ulocalized_time(
                    DateTime(),
                    context=self.context,
                    request=self.request
                ).lstrip('0')}
            )

class NewsletterForm(BaseCampaignForm):
    fields = field.Fields(
        INewsletter['subject'],
        INewsletter['interests'],
        INewsletter['schedule'],
        INewsletter['create_draft'],
    )

    fields['create_draft'].widgetFactory = SingleCheckBoxFieldWidget
    fields['interests'].widgetFactory = CheckBoxFieldWidget

    ignoreContext = True

    def getSegmentConditions(self, interest_groups):
        if not interest_groups and hasattr(self.context, 'interest_groups'):
            interest_groups = self.context.interest_groups
        if not interest_groups:
            return []

        groupings = {}
        for grouping_id, group_name, _ in interest_groups:
            groupings.setdefault(grouping_id, []).append(group_name)

        segments = []
        for grouping, interests in groupings.items():
            segments.append({
                'field': 'interests-' + str(grouping),
                'op': 'one',
                'value': ','.join(
                    [i.encode("utf-8") for i in interests]
                )
            })

        return segments

    @button.buttonAndHandler(_(u'Preview'))
    def handlePreview(self, action):
        if self.context.REQUEST.get('restricted_traverse', False):
            return

        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        url = self.context.aq_parent.absolute_url()
        return self.request.response.redirect(
            url + "/@@chimpfeed-newsletter-content")

    @button.buttonAndHandler(_(u'Create'))
    def handleCreate(self, action):
        if self.context.REQUEST.get('restricted_traverse', False):
            return

        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        if not self.context.select_interest_groups:
            data.update(dict(interests=self.context.interest_groups))
        success = self.process("campaignCreate", **data)

        if success:
            IStatusMessage(self.request).addStatusMessage(
                _(u"Campaign created."),
                "info"
            )

    def process(self, method, subject=None,
                create_draft=False, schedule=None, interests=None,
                **data):
        settings = IFeedSettings(self.context)
        api_key = settings.mailchimp_api_key

        view = getMultiAdapter(
            (self.context.aq_parent, self.request),
            name="chimpfeed-newsletter-campaign"
        )

        rendered = view.template().encode('utf-8')
        next_url = self.request.get('HTTP_REFERER') or self.action
        segment_conditions = self.getSegmentConditions(interests)

        return self.createCampaign(
            api_key, method, subject,
            create_draft, schedule, rendered, next_url,
            segment_conditions and {
                'match': 'any',
                'conditions': segment_conditions
            } or None
        )

    def update(self):
        super(NewsletterForm, self).update()

        today = datetime.date.today()

        subject = self.widgets['subject']
        if not subject.value:
            value = self.context.subject or \
                self.context.Title().decode('utf-8')

            subject.value = _(
                u"${subject} ${date}", mapping={
                'subject': value, 'date': ulocalized_time(
                    DateTime(),
                    context=self.context,
                    request=self.request
                ).lstrip('0')}
            )

        if not self.context.select_interest_groups:
            self.widgets['interests'].mode = HIDDEN_MODE

    def render(self):
        if self.context.REQUEST.get('restricted_traverse', False):
            return ''
        return BaseCampaignForm.render(self)


class ModerationForm(BaseForm):
    fields = field.Fields(IModeration)

    fields['items'].widgetFactory = ModerationWidget.factory

    ignoreContext = True

    @button.buttonAndHandler(_(u'Approve'))
    def handleApprove(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        catalog = self.context.portal_catalog
        bumped = []
        today = datetime.date.today()

        uids = []
        for rid in data['items'] or ():
            metadata = catalog.getMetadataForRID(rid)
            uids.append(metadata['UID'])

        settings = IFeedSettings(self.context)

        brains = catalog.unrestrictedSearchResults(UID=uids)
        for brain in brains:
            obj = brain.getObject()

            # Bump the scheduled date to today's date. This ensures that
            # the item will be shown on the moderation portlet.
            try:
                date = obj.getField('feedSchedule').get(obj)
                if date is not None:
                    date = date.asdatetime().date()
            except AttributeError:
                date = obj.feedSchedule

            if settings.bump_date_on_moderation and (date is None or date < today):
                try:
                    field = obj.getField('feedSchedule')
                except AttributeError:
                    obj.feedSchedule = today
                else:
                    field.set(obj, DateTime(
                        today.year, today.month, today.day
                    ))

                bumped.append(obj)

            try:
                field = obj.getField('feedModerate')
            except AttributeError:
                obj.feedModerate = True
            else:
                field.set(obj, True)

            # Reindex entire object (to make sure the metadata is
            # updated, too).
            obj.reindexObject()

        if data['items']:
            self.widgets['items'].update()

        if bumped:
            IStatusMessage(self.request).addStatusMessage(
                _(u"The scheduled date has been set to today's date "
                  u"for the following items that were scheduled to "
                  u"a date in the past: ${titles}.",
                  mapping={'titles': u', '.join(
                      [obj.Title().decode('utf-8') for obj in bumped])}),
                "info",
            )

    def update(self):
        super(ModerationForm, self).update()

        # No reason to show a button if no action is required (or
        # possible).
        if not self.widgets['items'].action_required:
            del self.actions['approve']


class SubscribeContext(Implicit):
    interest_groups = ()

    def __init__(self, request):
        list_id = self.mailinglist = request['list_id']
        self.factory = InterestGroupVocabulary(list_id)

    def get_name(self):
        api = getUtility(IApiUtility, context=self)
        for list_id, name in api.get_lists():
            if list_id == self.mailinglist:
                return name

        return _(u"Untitled")

    name = ComputedAttribute(get_name, 1)

    def get_interest_groupings(self):
        vocabulary = self.factory(self)
        return set(
            term.value[0] for term in vocabulary
        )

    interest_groupings = ComputedAttribute(get_interest_groupings, 1)


class SubscribeForm(BaseForm):

    @property
    def fields(self):
        fields = field.Fields()
        settings = IFeedSettings(self.context)
        if settings.show_name:
            fields += field.Fields(
                schema.TextLine(
                    __name__="name",
                    title=_(u"Name"),
                    required=False,
                )
            )
        fields += field.Fields(ISubscription)
        fields['interests'].widgetFactory = InterestsWidget.factory
        return fields

    @button.buttonAndHandler(_(u'Register'))
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        settings = IFeedSettings(self.context)
        api_key = settings.mailchimp_api_key
        next_url = self.nextURL()

        try:
            list_id = (
                data.pop('list_id', None) or
                self.getContent().mailinglist
            )
            try:
                name = data.pop('name')
            except KeyError:
                name = ''
            email = data.pop('email')
            interests = data.pop('interests')

            content = self.getContent()

            preselected = getattr(content, 'preselected_interest_groups', None)
            if preselected:
                if not interests:
                    interests = preselected
                else:
                    interests = interests + preselected

            if api_key:
                api = greatape.MailChimp(api_key, debug=False)

                # Negotiate language
                language = negotiate(self.request)

                # Split full name into first (given) and last name.
                try:
                    fname, lname = queryUtility(
                        INameSplitter, name=language,
                        default=GenericNameSplitter
                        ).split_name(name)
                except AttributeError:
                    fname, lname = u'', u''

                # Log subscription attempt.
                logger.info(("listSubscribe(%r, %r, %r, %r)" % (
                    list_id, email, fname, lname)).encode('utf-8'))

                merge_vars = {
                    'FNAME': fname.encode('utf-8'),
                    'LNAME': lname.encode('utf-8'),
                    'GROUPINGS': [
                        dict(
                            id=grouping_id,
                            groups=",".join(
                                group.
                                encode('utf-8').
                                replace(',', '\\,')
                                for group in group_names
                            ),
                        )
                        for (grouping_id, group_names) in
                        create_groupings(interests).items()
                    ]
                }

                for name, value in data.items():
                    if value is not None:
                        merge_vars[name.upper()] = value.encode('utf-8')

                try:
                    result = api(
                        method="listSubscribe", id=list_id,
                        email_address=email,
                        update_existing=True,
                        replace_interests=False,
                        merge_vars=merge_vars
                    )
                except greatape.MailChimpError, exc:
                    logger.warn(exc.msg)

                    # ... is not a valid interest grouping id for the list
                    if exc.code == 270:
                        return IStatusMessage(self.request).addStatusMessage(
                            _(u"There was a problem signing you up for "
                              u"the selected interest groups. This could "
                              u"mean that the subscription service is "
                              u"configured incorrectly. Please contact "
                              u"the webmaster."),
                            "error",
                        )
                else:
                    if result:
                        next_url += ('?' in next_url
                                     and '&' or '?') + 'success=yes'

                        return IStatusMessage(self.request).addStatusMessage(
                            _(u"Thank you for signing up. We'll send you a "
                              u"confirmation message by e-mail shortly."),
                            "info"
                        )

            IStatusMessage(self.request).addStatusMessage(
                _(u"An error occurred while processing your "
                  u"request to sign up for ${email}. "
                  u"Please try again!", mapping={'email': email}),
                "error")

        finally:
            self.request.response.redirect(next_url)

    def nextURL(self):
        return self.action


class JavascriptWidget(field.Field):
    """Generic javascript snippet."""

    error = None
    required = False

    def __call__(self, field, request):
        widget = type(self)(field)
        widget.request = request
        alsoProvides(widget, IWidget)
        return widget

    def get(self, mode):
        return self

    @property
    def widgetFactory(self):
        return self

    def render(self):
        return u'<script type="text/javascript">\n' + \
               self.body + \
               u'</script>'

    def update(self):
        assert self.mode == HIDDEN_MODE

    def extract(self):
        return NO_VALUE


class SelectAllGroupsJavascript(JavascriptWidget):
    """Replace the fieldset legend with a select all checkbox."""

    path = os.path.join(os.path.dirname(__file__), "select.js")

    @property
    @forever
    def body(self):
        body = open(self.path, 'rb').read()

        # Extract and translate strings by heuristic:
        td = getUtility(ITranslationDomain, name="collective.chimpfeed")
        for string in re.compile(r'>([\w\s]+)<').findall(body):
            msg_id = string.decode('utf-8')
            translation = td.translate(msg_id, context=self.request)
            body = body.replace(string, translation)

        return body


class ListSubscribeForm(SubscribeForm):
    @property
    def fields(self):
        fields = field.Fields()
        settings = IFeedSettings(self.context)

        # Javascript-widget
        if settings.enable_select_all:
            fields += field.Fields(SelectAllGroupsJavascript(schema.Field(
                __name__="js", required=False), mode="hidden"))

        # Include form fields, but change the order around.
        fields += field.Fields(ISubscription).select('interests')
        fields['interests'].widgetFactory = InterestsWidget.factory

        if settings.show_name:
            fields += field.Fields(
                schema.TextLine(
                    __name__="name",
                    title=_(u"Name"),
                    required=False,
                )
            )

        fields += field.Fields(ISubscription).select('email')

        # Add mailinglist as hidden field
        fields += field.Fields(schema.ASCII(
            __name__="list_id",
            required=True),
            mode="hidden"
        )

        context = self.getContent()
        api = getUtility(IApiUtility, context=self.context)
        result = api.list_merge_vars(context.mailinglist)

        for entry in result:
            name = entry['tag'].lower()

            if name in fields:
                continue

            if not entry['show']:
                continue

            # Skip all-uppercase:
            if entry['name'] == entry['name'].upper():
                continue

            field_type = entry['field_type']
            required = entry['req']

            if field_type == 'text':
                factory = schema.TextLine
                options = {}

            elif field_type == 'dropdown':
                factory = schema.Choice
                choices = list(entry['choices'])

                if not required:
                    choices.append(u"")
                    required = True

                options = {
                    'vocabulary': SimpleVocabulary([
                        SimpleTerm(value=value,
                                   token=value.encode(
                                       'ascii',
                                       'xmlcharrefreplace'),
                                   title=value or _(u"None"))
                        for value in choices])
                }

            else:
                continue

            fields += field.Fields(factory(
                __name__=name.encode('utf-8'),
                default=entry['default'],
                required=required,
                title=_(entry['name']),
                **options
            ))

        return fields

    # Remove prefix; we want to be able to provide defaults using a
    # simple format.
    prefix = ""

    @property
    def description(self):
        context = self.context.aq_base
        if self.request.get('success'):
            return u''
        elif not IPloneSiteRoot.providedBy(context) \
                 and context.Description():
            return context.Description()
        else:
            return _(u"Select subscription options and submit form.")

    @property
    def label(self):
        if self.request.get('success'):
            return _(u"Request processed")

        name = self.getContent().name
        return _(u"Subscribe to: ${name}", mapping={'name': name})

    @view.memoize
    def getContent(self):
        if ISubscriptionFormSettings.providedBy(self.context):
            return self.context

        settings = IFeedSettings(self.context)
        try:
            context = SubscribeContext(self.request)
        except KeyError, exc:
            raise BadRequest(u"Missing parameter: %s." % exc)

        return context.__of__(
            ImplicitAcquisitionWrapper(
                settings, self.context)
        )

    def nextURL(self):
        list_id = self.request.get('list_id', '')
        return self.action + "?list_id=%s" % list_id

    def updateActions(self):
        self.actions = getMultiAdapter(
            (self, self.request, self.getContent()), IActions)

        self.actions.update()

        if self.request.get('success'):
            for name in self.actions:
                del self.actions[name]

    def updateWidgets(self):
        self.widgets = getMultiAdapter(
            (self, self.request, self.getContent()), IWidgets
        )

        if self.request.get('success'):
            mode = HIDDEN_MODE

            for name, widget in self.widgets.items():
                if isinstance(widget, JavascriptWidget):
                    del self.widgets[name]
        else:
            mode = self.mode

        self.widgets.mode = mode
        self.widgets.prefix = ""
        self.widgets.ignoreContext = self.ignoreContext
        self.widgets.ignoreRequest = self.ignoreRequest
        self.widgets.ignoreReadonly = self.ignoreReadonly
        self.widgets.update()

ListSubscribeFormView = wrap_form(ListSubscribeForm)


class ListSubscribeFormFieldWidgets(field.FieldWidgets):
    prefix = ""



from logging import getLogger

log = getLogger('collective.chimpfeed:upgrades')

def runProfiles(site, profile_ids):
    """ """
    setup_tool = getattr(site, 'portal_setup')

    for profile_id in profile_ids:
        if hasattr(setup_tool, 'runAllImportStepsFromProfile'):
            if not profile_id.startswith('profile-'):
                profile_id = "profile-%s" % profile_id
            setup_tool.runAllImportStepsFromProfile(profile_id)
        else:
            setup_tool.setImportContext(profile_id)
            setup_tool.runAllImportSteps()
        log.info("Ran profile " + profile_id)

def runDefaultProfile(tool):
    """ Run default profile """
    site = tool.aq_parent
    runProfiles(site, ('collective.chimpfeed:default',))



import greatape

from zope.component import getUtilitiesFor

from Products.AdvancedQuery import Indexed, Ge, Le, Eq, In
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from Products.CMFCore.utils import getToolByName

from DateTime import DateTime

from collective.chimpfeed.interfaces import IFeedSettings
from collective.chimpfeed.interfaces import IGroupSorter
from collective.chimpfeed.interfaces import IGroupExtras
from collective.chimpfeed import MessageFactory as _

from collective.chimpfeed.vocabularies import InterestGroupVocabulary

from zope.component import queryUtility
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class CampaignView(BrowserView):
    template = ViewPageTemplateFile('campaign.pt')

    title = _(u"Preview")

    def results(self, start, until=None):
        today = DateTime()
        today = DateTime(today.year(), today.month(), today.day())
        start = DateTime(start)
        start = DateTime(start.year(), start.month(), start.day())

        query = Indexed('chimpfeeds') & \
                In('review_state', ('published', )) & \
                Ge('feedSchedule', start)

        if until:
            try:
                until = DateTime(until)
            except DateTime.SyntaxError:
                pass
            else:
                query = query & Le('feedSchedule', until)

        site = getToolByName(self.context, "portal_url").getPortalObject()
        settings = IFeedSettings(site)
        if settings.use_moderation:
            query = query & Eq('feedModerate', True)

        catalog = getToolByName(self.context, "portal_catalog")

        extras = []
        utilities = getUtilitiesFor(IGroupExtras)
        groups = InterestGroupVocabulary()(self.context)
        for name, util in utilities:
            for group in groups:
                extras.extend(util.items(group.title, start, until))

        return list(catalog.evalAdvancedQuery(
            query, (('feedSchedule', 'desc'), ))) + extras
        

    def getGroupings(self, start, until=None):
        groupings = {}
        for brain in self.results(start, until=until):

            # Note that an item can appear in more than one group.
            categories = [name.rsplit(':', 1)[0] for name in brain.chimpfeeds]
            for category in set(categories):
                items = groupings.setdefault(category, [])
                items.append(brain)

        sorting = queryUtility(IGroupSorter)
        if sorting is None:
            key = lambda name, items: name
        else:
            key = sorting.key

        return sorted(groupings.items(), key=lambda item: key(*item))

    def getSegments(self, start, until=None, **kwargs):
        chimpfeeds = set()
        for brain in self.results(start, until=until):
            chimpfeeds.update(
                (feed.replace(' ', '') for feed in brain.chimpfeeds)
            )

        segments = {}
        
        for term in InterestGroupVocabulary()(self.context):
            if term.title.replace(' ', '') in chimpfeeds:
                groupingid, grouptitle, groupid = term.value
                items = segments.setdefault(groupingid, [])
                items.append(groupid)
                
        return segments

class CampaignContentView(BrowserView):
    title = _(u"Campaign created")

    def template(self):
        site = getToolByName(self.context, "portal_url").getPortalObject()
        settings = IFeedSettings(site)
        api_key = settings.mailchimp_api_key

        if not api_key:
            return

        try:
            api = greatape.MailChimp(api_key, debug=False)
            cid = self.request.get('cid', '')
            content = api(method="campaignContent", cid=cid)
        except greatape.MailChimpError:
            IStatusMessage(self.request).addStatusMessage(
                _(u"Unable to show content."), "error"
                )
            return

        return content['html']



from Products.CMFCore.permissions import setDefaultRoles

MODERATE_PERMISSION = "collective.chimpfeed: Moderate"
SCHEDULE_PERMISSION = "collective.chimpfeed: Schedule"
CAMPAIGN_PERMISSION = "collective.chimpfeed: Send or schedule campaign"

setDefaultRoles(MODERATE_PERMISSION, ('Manager', ))
setDefaultRoles(SCHEDULE_PERMISSION, ('Manager', ))
setDefaultRoles(CAMPAIGN_PERMISSION, ('Manager', ))



 # encoding: utf-8
from Products.CMFCore.utils import getToolByName

catalog_indexes = (
    {'name'  : 'chimpfeeds',
     'type'  : 'KeywordIndex',
     },
    {'name'  : 'feedSchedule',
     'type'  : 'DateIndex',
     },
    {'name'  : 'feedModerate',
     'type'  : 'FieldIndex',
     },
    )
catalog_columns = ('chimpfeeds', 'feedCategory', 'feedSchedule', 'feedModerate',)
purge_existing_indexes = ()
purge_existing_columns = ()

def setup_indexes(portal, indexes, columns, purge_indexes=(),
                  purge_columns=()):
    ct = getToolByName(portal, 'portal_catalog')

    new_metadata = False

    for idx in indexes:
        if idx['name'] in purge_indexes and idx['name'] in ct.indexes():
            ct.delIndex(idx['name'])
        if idx['name'] in ct.indexes():
            print "Found the '%s' index in the catalog, nothing changed.\n" % \
                  idx['name']
        else:
            ct.addIndex(**idx)
            print "Added '%s' (%s) to the catalog.\n" % \
                  (idx['name'], idx['type'])
    for entry in columns:
        if entry in purge_columns and entry in ct.schema():
            ct.delColumn(entry)
        if entry in ct.schema():
            print "Found '%s' in the catalog metadatas, nothing changed.\n" % \
                  entry
        else:
            ct.addColumn(entry)
            print "Added '%s' to the catalog metadatas.\n" % entry
            new_metadata = True
    if new_metadata:
        print "=== portal_catalog may need re-indexing. ===\n"

def setupVarious(context):

    if context.readDataFile('collective.chimpfeed.txt') is None:
        return

    # Create indexes
    setup_indexes(portal=context.getSite(),
                  indexes=catalog_indexes,
                  columns=catalog_columns)



from z3c.form import form
from z3c.form import button

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage

from zope.annotation.interfaces import IAnnotations
from zope.interface import classImplements
from plone.z3cform import layout

from collective.chimpfeed import MessageFactory as _

ANNOTATION_KEY = 'collective.chimpfeed'


def annotation_proxy(context, schema):
    annotations = IAnnotations(context)
    properties = dict(
        (name, AnnotationProperty(annotations, schema[name]))
        for name in schema
        )

    properties['portal_url'] = staticmethod(context.portal_url)

    proxy = type(
        "%sAdapter" % schema.__name__[1:],
        (object, ), properties,
        )

    classImplements(proxy, schema)

    return proxy()


class AnnotationProperty(object):
    def __init__(self, annotations, field):
        self.annotations = annotations
        self.field = field

    def __get__(self, inst, owner):
        annotation = self.annotations.get(ANNOTATION_KEY, {})
        return annotation.get(self.field.__name__, self.field.default)

    def __set__(self, inst, value):
        annotation = self.annotations.get(ANNOTATION_KEY, {})
        annotation[self.field.__name__] = value
        self.annotations[ANNOTATION_KEY] = annotation


class RegistryEditForm(form.EditForm):
    control_panel_view = "plone_control_panel"

    def getContent(self):
        return annotation_proxy(self.context, self.schema)

    @button.buttonAndHandler(_(u"Save"), name='save')
    def handleSave(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        self.applyChanges(data)
        IStatusMessage(self.request).addStatusMessage(
            _(u"Changes saved."), "info"
            )

        self._redirect()

    @button.buttonAndHandler(_(u"Cancel"), name='cancel')
    def handleCancel(self, action):
        IStatusMessage(self.request).addStatusMessage(
            _(u"Edit cancelled."), "info"
            )

        self._redirect()

    def _redirect(self):
        self.request.response.redirect(
            "%s/%s" % (self.context.absolute_url(),
                       self.control_panel_view)
            )


class ControlPanelFormWrapper(layout.FormWrapper):
    index = ViewPageTemplateFile('controlpanel.pt')



import logging

from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage

from Acquisition import ImplicitAcquisitionWrapper

from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.interface import implements
from zope.component import adapts
from zope.lifecycleevent import modified
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from plone.z3cform import layout

try:
    from plone.app.registry.browser import controlpanel
    switch_on = None
except ImportError:
    from collective.chimpfeed import legacy as controlpanel
    from plone.z3cform.z2 import switch_on

from collective.chimpfeed.interfaces import IFeedSettings
from collective.chimpfeed.interfaces import IControlPanel
from collective.chimpfeed import MessageFactory as _
from collective.chimpfeed.feeds import make_urls
from collective.chimpfeed.vocabularies import feeds_factory
from collective.chimpfeed.vocabularies import lists_factory

from z3c.form import button
from z3c.form import field
from z3c.form import widget
from z3c.form.interfaces import IDataConverter

from Products.AdvancedQuery import Indexed

try:
    from z3c.form.browser import textlines
except ImportError:
    logging.warn("z3c.form library does not have textlines widget.")
    textlines = None

logger = logging.getLogger("collective.chimpfeed.controlpanel")


class ControlPanelAdapter(object):
    adapts(IFeedSettings)
    implements(IControlPanel)

    def __init__(self, settings):
        self.settings = settings

    @property
    def urls(self):
        portal_url = getToolByName(self.settings, 'portal_url')()

        vocabulary = feeds_factory(self.settings)

        # Use term's titles as the basis for the feed URLs.
        urls = make_urls(vocabulary)

        return tuple(
            ("%s/++chimpfeeds++/%s" % (portal_url, url), title)
            for url, value, title in urls
            )

    @property
    def lists(self):
        portal_url = getToolByName(self.settings, 'portal_url')()
        vocabulary = lists_factory(self.settings)

        return tuple(
            ("%s/@@chimpfeed-subscribe?list_id=%s" % (
                portal_url, term.value), term.title)
            for term in sorted(vocabulary, key=lambda term: term.title)
            )



class UrlsWidget(widget.Widget):
    implements(IDataConverter)

    render = ViewPageTemplateFile("urls.pt")

    @classmethod
    def factory(cls, field, request):
        return widget.FieldWidget(field, cls(request))

    def toWidgetValue(self, value):
        return value


class ControlPanelEditForm(controlpanel.RegistryEditForm):
    schema = IFeedSettings
    fields = field.Fields(IControlPanel)

    label = _(u"MailChimp settings")
    description = _(
        u"Please configure an API-key and define one or more "
        u"feeds."
        )

    fields['urls'].mode = "display"
    fields['urls'].widgetFactory = UrlsWidget.factory

    fields['lists'].mode = "display"
    fields['lists'].widgetFactory = UrlsWidget.factory

    if textlines is not None:
        fields['feeds'].widgetFactory = textlines.TextLinesFieldWidget

    buttons = button.Buttons()
    buttons += controlpanel.RegistryEditForm.buttons
    handlers = controlpanel.RegistryEditForm.handlers.copy()

    def updateActions(self):
        # This prevents a redirect to the main control panel page
        self.request.response.setStatus(200, lock=True)

        super(ControlPanelEditForm, self).updateActions()

    def render(self):
        if switch_on is not None:
            switch_on(self)

        urls = self.widgets['urls']
        urls.update()

        if not urls.value:
            del self.widgets['urls']

        return super(ControlPanelEditForm, self).render()

    def getContent(self):
        content = super(ControlPanelEditForm, self).getContent()
        return ImplicitAcquisitionWrapper(content, self.context)

    @button.buttonAndHandler(_(u"Repair"), name='repair')
    def handleRepair(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        query = Indexed('chimpfeeds')
        brains = self.context.portal_catalog.evalAdvancedQuery(query)

        context = self.getContent()
        vocabulary = feeds_factory(context)
        all_feeds = set(term.value for term in vocabulary)

        count = 0
        bad = set()

        changed = []
        for i, brain in enumerate(brains):
            try:
                feeds = set(brain.chimpfeeds)
            except TypeError:
                continue

            missing = feeds - all_feeds
            bad |= missing

            if missing:
                count += 1
                obj = brain.getObject()
                try:
                    field = obj.getField('feeds')
                except AttributeError:
                    feeds = obj.feeds
                    field = None
                else:
                    feeds = set(field.get(obj))

                fixed = feeds - missing

                if field is None:
                    obj.feeds = fixed
                else:
                    field.set(obj, fixed)

                changed.append(obj)

        for obj in changed:
            modified(obj)
            obj.reindexObject()

        logger.info("Repair complete; %d items updated." % count)
        if bad:
            logger.info("Feeds removed: %s." % (", ".join(bad).encode('utf-8')))

        IStatusMessage(self.request).addStatusMessage(
            _(u"Repaired ${count} items.", mapping={'count': count}),
            "info"
        )


ControlPanel = layout.wrap_form(
    ControlPanelEditForm,
    controlpanel.ControlPanelFormWrapper
    )



from zope.interface import implements
from zope.component import getUtility
from zope.component import ComponentLookupError
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
try:
 # Plone < 4.3
 from zope.app.component.hooks import getSite
except ImportError:
 # Plone >= 4.3
 from zope.component.hooks import getSite  # NOQA 

from plone.i18n.normalizer.interfaces import IIDNormalizer

from collective.chimpfeed.interfaces import IFeedSettings, IApiUtility
from collective.chimpfeed import logger
from collective.chimpfeed import MessageFactory as _

from Products.AdvancedQuery import Indexed, Ge, Eq
from DateTime import DateTime
from Acquisition import ImplicitAcquisitionWrapper


class VocabularyBase(object):
    implements(IVocabularyFactory)


class ScheduledItems(VocabularyBase):
    def __call__(self, context):
        terms = []
        today = DateTime()
        today = DateTime(today.year(), today.month(), today.day())

        query = (
            Indexed('chimpfeeds') &
            Eq('allowedRolesAndUsers', 'Anonymous') &
            (Ge('feedSchedule', today) | Eq('feedModerate', False))
        )

        brains = context.portal_catalog.evalAdvancedQuery(
            query, (('feedSchedule', 'desc'),))

        for brain in brains:
            rid = brain.getRID()
            terms.append(
                SimpleTerm(rid, rid, brain.Title)
            )

        return SimpleVocabulary(terms)


class SettingVocabulary(VocabularyBase):
    field = None

    def __call__(self, context):
        normalize = getUtility(IIDNormalizer).normalize
        settings = IFeedSettings(context)
        name = self.field.__name__

        return SimpleVocabulary([
            SimpleTerm(feed, normalize(feed), feed)
            for feed in getattr(settings, name, None) or ()
            ])


class CategoryVocabulary(SettingVocabulary):
    field = IFeedSettings['categories']


class MailChimpVocabulary(VocabularyBase):
    def __call__(self, context):

        try:
            utility = getUtility(IApiUtility, context=context)
        except ComponentLookupError:
            site = getSite()
            utility = getUtility(IApiUtility, context=site)
        else:
            utility = utility.__of__(context)

        wrapped = ImplicitAcquisitionWrapper(self, utility)

        generator = wrapped.get_terms()
        terms = tuple(generator)

        try:
            return SimpleVocabulary(terms)
        except ValueError as e:
            logger.info(e.message)
            return SimpleVocabulary([])


class ListVocabulary(MailChimpVocabulary):
    def get_terms(self):
        for value, name in self.get_lists():
            yield SimpleTerm(value, value, name)


class TemplateVocabulary(MailChimpVocabulary):
    def get_terms(self):
        yield SimpleTerm(u"", "", _(u"Empty"))

        for value, name in self.get_templates():
            yield SimpleTerm(value, value, name)


class InterestGroupVocabulary(MailChimpVocabulary):
    def __init__(self, list_id=None):
        self.list_id = list_id

    def get_terms(self):
        groupings = self.get_groupings(
            self.list_id or
            getattr(self, "mailinglist", None) or
            IFeedSettings(self).mailinglist
        )

        for grouping in groupings:
            for term in self.get_terms_for_grouping(grouping, True):
                yield term

    def get_terms_for_grouping(self, grouping, qualified=False):
        terms = []
        groups = sorted(grouping['groups'],
                        key=lambda group: group['display_order'])
        for group in groups:
            terms.append(self.get_term_for_group(group, grouping, qualified))
        return terms

    def get_term_value(self, group, grouping):
        return (grouping['id'], group['name'], group['bit'])

    def get_term_for_group(self, group, grouping, qualified=False):
        name = group['name']
        value = self.get_term_value(group, grouping)

        normalize = getUtility(IIDNormalizer).normalize
        token = "%s-%s" % (grouping['id'], normalize(name))

        if qualified:
            name = grouping['name'] + " : " + name

        return SimpleTerm(value, token, name)


class InterestGroupingVocabulary(MailChimpVocabulary):
    def get_terms(self):
        groupings = self.get_groupings()

        for grouping in groupings:
            value = token = grouping['id']
            groups = grouping['groups']
            name = grouping['name']

            if groups:
                names = [group['name'] for group in groups]

                if len(names) > 4:
                    del names[4:]

                    name = _(
                        u"${name} (${groups} and ${count} more)",
                        mapping={
                            'name': name,
                            'groups': ", ".join(names),
                            'count': len(groups) - len(names),
                            }
                        )
                else:
                    name += u" (%s)" % ", ".join(names)

            yield SimpleTerm(value, token, name)

class FeedVocabulary(InterestGroupVocabulary):

    def get_term_value(self, group, grouping):
        return "%s:%s" % (
            grouping['name'].replace(':', ''),
            group['name'].replace(':', '')
            )

    def get_terms(self):
        terms = list(super(FeedVocabulary, self).get_terms())

        feeds = IFeedSettings(self).feeds

        if feeds:
            for feed in feeds:
                terms.append(SimpleTerm(feed, feed.encode('utf-8'), feed))

        return sorted(terms, key=lambda term: term.title)

    def get_groupings(self, list_id=None):
        groupings = self.aq_parent.get_groupings(list_id=list_id)

        if IFeedSettings(self).ignored_groupings:
            return [g for g in groupings if g['id'] not in \
                    IFeedSettings(self).ignored_groupings]
        return groupings



lists_factory = ListVocabulary()
interest_groupings_factory = InterestGroupingVocabulary()
interest_groups_factory = InterestGroupVocabulary()
feeds_factory = FeedVocabulary()
scheduled_items = ScheduledItems()
categories_factory = CategoryVocabulary()
templates = TemplateVocabulary()



import os
from setuptools import setup, find_packages


def read(*pathnames):
    return open(os.path.join(os.path.dirname(__file__), *pathnames)).read()

version = '2.0'

setup(name='collective.chimpfeed',
      version=version,
      description="MailChimp-integration for Plone!",
      long_description='\n'.join([
          read('README.rst'),
          read('CHANGES.rst'),
          ]),
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Programming Language :: Python",
        ],
      keywords='plone rss mailchimp',
      author='Malthe Borch',
      author_email='mborch@gmail.com',
      license='GPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'plone.indexer',
          'plone.z3cform',
          'greatape',
          'simplejson',
          'Products.AdvancedQuery',
          'BeautifulSoup',
          'zope.app.component',
      ],
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



import sys

from Products.Five.testbrowser import Browser
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup
from Products.Five import zcml
from Products.Five import fiveconfigure
from Testing import ZopeTestCase as ztc

packages = (
    'collective.chimpfeed',
    )


@onsetup
def setup_product():
    fiveconfigure.debug_mode = True

    for package in packages:
        __import__(package)
        module = sys.modules[package]

    zcml.load_config('configure.zcml', module)

    fiveconfigure.debug_mode = False

    for package in packages:
        ztc.installPackage(package)

setup_product()
ptc.setupPloneSite(products=packages)


class BaseFunctionalTestCase(ptc.FunctionalTestCase):
    def afterSetUp(self):
        ptc.FunctionalTestCase.afterSetUp(self)

        self.browser = Browser()
        self.browser.handleErrors = False
        self.portal.error_log._ignored_exceptions = ()

        def raising(self, info):
            import traceback
            traceback.print_tb(info[2])
            print info[1]

        from Products.SiteErrorLog.SiteErrorLog import SiteErrorLog
        SiteErrorLog.raising = raising

    def loginAsAdmin(self):
        from Products.PloneTestCase.setup import portal_owner, default_password
        browser = self.browser
        browser.open(self.portal_url + "/login_form")
        browser.getControl(name='__ac_name').value = portal_owner
        browser.getControl(name='__ac_password').value = default_password
        browser.getControl(name='submit').click()

    @property
    def portal_url(self):
        return self.portal.absolute_url()


class PortletTest(BaseFunctionalTestCase):
    def clickAddPortlet(self):
        browser = self.browser
        browser.open(self.portal_url)
        browser.getLink('Manage portlets').click()
        right_column = browser.getForm(index=3)
        right_column.getControl('Subscription portlet').selected = True
        right_column.submit()

    def test_add_portlet(self):
        self.loginAsAdmin()
        self.clickAddPortlet()
        self.browser.getControl('Save').click()



from zope.component import ComponentLookupError
from collective.chimpfeed.interfaces import IFeedSettings

try:
    from plone.registry.interfaces import IRegistry
except ImportError:
    from collective.chimpfeed.legacy import annotation_proxy

    def get_settings(context):
        site = context.portal_url.getPortalObject()
        return annotation_proxy(site, IFeedSettings)
else:
    from zope.component import getUtility

    def get_settings(context):
        try:
            return getUtility(IRegistry).forInterface(IFeedSettings)
        except KeyError:
            raise ComponentLookupError(IFeedSettings)



import greatape
import time

from collective.chimpfeed import logger
from collective.chimpfeed.interfaces import IApiUtility, IFeedSettings
from zope.interface import implements

from Acquisition import Implicit, aq_parent

from plone.memoize.ram import cache

def stub_api(**kwargs):
    return ()

def make_cache_key(minutes):
    def cache_key(func, inst, *args):
        api_key = inst.get_api_key()
        return time.time() // (60 * minutes), api_key, args
    return cache_key


class ApiUtility(Implicit):
    implements(IApiUtility)

    def get_api_key(self):
        settings = IFeedSettings(aq_parent(self), None)
        if settings:
            return settings.mailchimp_api_key

    def api(self, **kwargs):
        api_key = self.get_api_key()

        if api_key:
            api = greatape.MailChimp(api_key)

            def api(api=api, **kwargs):
                logger.info("mailchimp(%s)" % (", ".join(
                    "%s=%r" % item for item in kwargs.items())))

                try:
                    return api(**kwargs)
                except greatape.MailChimpError, exc:
                    # http://apidocs.mailchimp.com/api/1.3/exceptions.field.php
                    if exc.code <= 0:
                        logger.critical(exc.msg)
                    elif exc.code < 120:
                        logger.warn(exc.msg)
                    elif exc.code < 200:
                        logger.info(exc.msg)
                except TypeError, exc:
                    logger.warn(exc.msg)

                return ()

        else:
            api = stub_api

        return api(**kwargs)

    @cache(make_cache_key(60))
    def _get_cached_groupings(self, list_id):
        results = []

        if list_id is None:
            list_ids = [list_id for (list_id, list_name) in self.get_lists()]
        else:
            list_ids = [list_id]

        for list_id in list_ids:
            groupings = self.api(
                method="listInterestGroupings", id=list_id
            )
            for grouping in groupings:
                results.append(grouping)

        return results

    @cache(make_cache_key(15))
    def get_lists(self):
        results = []
        for result in self.api(method="lists"):
            results.append((result['id'], result['name']))

        return results

    def get_groupings(self, list_id=None):
        if list_id is None:
            list_id = getattr(self, 'mailinglist', None)

        return self._get_cached_groupings(list_id)

    @cache(make_cache_key(60))
    def get_templates(self):
        results = []
        for result in self.api(method="campaignTemplates"):
            name = result['name']

            if name == 'Untitled Template':
                continue

            results.append(
                (result['id'], "%s (%s)" % (
                    name,
                    ", ".join(result['sections']))))

        return results

    @cache(make_cache_key(60))
    def list_merge_vars(self, list_id):
        return self.api(method="listMergeVars", id=list_id)





import logging
import weakref

from zope.interface import implements
from zope.interface import alsoProvides
from zope.component import adapts
from archetypes.schemaextender.interfaces import ISchemaExtender, IBrowserLayerAwareExtender
from archetypes.schemaextender.field import ExtensionField
from plone.indexer.decorator import indexer

from Acquisition.interfaces import IAcquirer
from Acquisition import aq_base

from Products.Archetypes import atapi
from Products.Archetypes.interfaces import IBaseContent
from DateTime import DateTime

from collective.chimpfeed.permissions import MODERATE_PERMISSION
from collective.chimpfeed.permissions import SCHEDULE_PERMISSION
from collective.chimpfeed.interfaces import IFeedControl
from collective.chimpfeed.interfaces import IFeedSettings
from collective.chimpfeed.interfaces import IBrowserLayer

try:
    from plone.app.dexterity.behaviors.metadata import Categorization
    from plone.app.dexterity.behaviors.metadata import DCFieldProperty
except ImportError:
    pass
else:
    class FeedControl(Categorization):
        feeds = DCFieldProperty(IFeedControl['feeds'])
        feedCategory = DCFieldProperty(IFeedControl['feedCategory'])
        feedSchedule = DCFieldProperty(IFeedControl['feedSchedule'])
        feedModerate = DCFieldProperty(IFeedControl['feedModerate'])


try:
    from plone.autoform.interfaces import IFormFieldProvider
except ImportError:
    pass
else:
    alsoProvides(IFeedControl, IFormFieldProvider)


@indexer(IBaseContent)
def at_feed_indexer(context):
    feeds = context.getField('feeds').get(context)
    return tuple(unicode(feed, 'utf-8') for feed in feeds)


@indexer(IBaseContent)
def at_schedule_indexer(context):
    schedule = context.getField('feedSchedule').get(context)
    if schedule is None:
        return context.getField('effectiveDate').get(context)

    return schedule

@indexer(IBaseContent)
def at_category_indexer(context):
    category = context.getField('feedCategory').get(context)
    if category is not None:
        return unicode(category, 'utf-8')


try:
    from plone.dexterity.interfaces import IDexterityContent
except ImportError:
    pass
else:
    from plone.behavior.interfaces import IBehaviorAssignable
    from plone.app.dexterity.behaviors.metadata import IDublinCore

    @indexer(IDexterityContent)
    def dx_schedule_indexer(context):
        assignable = IBehaviorAssignable(context, None)
        if assignable is None or not assignable.supports(IFeedControl):
            return

        date = getattr(context, "feedSchedule", None)
        if date is None:
            assignable = IBehaviorAssignable(context, None)
            if assignable is not None:
                if assignable.supports(IDublinCore):
                    return context.effective_date

        return DateTime(
            date.year,
            date.month,
            date.day
            )

    @indexer(IDexterityContent)
    def dx_feed_indexer(context):
        assignable = IBehaviorAssignable(context, None)
        if assignable is not None:
            if assignable.supports(IFeedControl):
                try:
                    return tuple(context.feeds)
                except TypeError:
                    return tuple()

    @indexer(IDexterityContent)
    def dx_category_indexer(context):
        assignable = IBehaviorAssignable(context, None)
        if assignable is not None:
            if assignable.supports(IFeedControl):
                return getattr(context, "feedCategory", None)


class StringField(ExtensionField, atapi.StringField):
    pass


class ScheduleField(ExtensionField, atapi.DateTimeField):
    def set(self, instance, value, **kwargs):
        previous = self.get(instance, **kwargs)

        # Set moderation to `False` to prompt an editor to moderate
        # new date.
        if previous == value:
            return

        storage = instance.getField('feedModerate').getStorage()
        storage.set('feedModerate', instance, False, **kwargs)

        super(ScheduleField, self).set(instance, value, **kwargs)


class LinesField(ExtensionField, atapi.LinesField):
    pass


class ModerationField(ExtensionField, atapi.BooleanField):
    def set(self, instance, value, **kwargs):
        schedule = instance.getField('feedSchedule')
        super(ModerationField, self).set(instance, value, **kwargs)

        date = schedule.get(instance)
        if date is None:
            return


class FeedExtender(object):
    implements(ISchemaExtender, IBrowserLayerAwareExtender)
    adapts(IBaseContent)

    layer = IBrowserLayer

    fields = (
        LinesField(
            name="feeds",
            required=False,
            multivalued=1,
            schemata="settings",
            widget=atapi.MultiSelectionWidget(
                label=IFeedControl['feeds'].title,
                description=IFeedControl['feeds'].description,
                visible={ "view" : "invisible" },
                ),
            vocabulary_factory=IFeedControl['feeds'].value_type.vocabularyName,
            ),

        StringField(
            'feedCategory',
            schemata="settings",
            languageIndependent=True,
            required=False,
            default=None,
            widget=atapi.SelectionWidget(
                label=IFeedControl['feedCategory'].title,
                description=IFeedControl['feedCategory'].description,
                visible={ "view" : "invisible" },
                ),
            vocabulary_factory=IFeedControl['feedCategory'].vocabularyName,
            ),

        ScheduleField(
            'feedSchedule',
            schemata="settings",
            languageIndependent=True,
            required=False,
            default=None,
            write_permission=SCHEDULE_PERMISSION,
            widget=atapi.CalendarWidget(
                show_hm=False,
                starting_year=2012,
                ending_year=2015,
                label=IFeedControl['feedSchedule'].title,
                description=IFeedControl['feedSchedule'].description,
                visible={ "view" : "invisible" },
                ),
            ),

        ModerationField(
            'feedModerate',
            schemata="settings",
            default=False,
            enforceVocabulary=1,
            write_permission=MODERATE_PERMISSION,
            widget=atapi.BooleanWidget(
                label=IFeedControl['feedModerate'].title,
                description=IFeedControl['feedModerate'].description,
                visible={ "view" : "invisible" },
                ),
            ),
        )

    types = weakref.WeakKeyDictionary()

    def __init__(self, context):
        self.context = context

    def getFields(self):
        if IAcquirer.providedBy(self.context):
            base = aq_base(self.context)
        else:
            base = self.context

        cls = type(base)
        applicable = self.types.get(cls)
        if applicable is None:
            # Check that we can get a settings object for this
            # context.
            applicable = IFeedSettings(self.context, None) is not None

            if applicable:
                # If there's an overlap on field names, we do not extend
                # the content schema. Note that Archetypes allows
                # overriding a field which is why we need to perform this
                # check ourselves.
                names = set(field.__name__ for field in self.fields)
                existing = self.context.schema.keys()
                overlap = names & set(existing)
                applicable = not bool(overlap)

            self.types[cls] = applicable

            if not applicable:
                logging.getLogger('collective.chimpfeed').info(
                    "skipping %s." % cls.__name__
                    )

        if not applicable:
            return ()

        return self.fields
